@@ -166,7 +166,7 @@ class ROOTDirectory(object):
 
                 return ROOTDirectory.read(source, Cursor(fBEGIN + fNbytesName), context, mykey)
 
-            except:
+            except Exception:
                 source.dismiss()
                 raise
 
@@ -362,6 +362,9 @@ class ROOTDirectory(object):
             else:
                 raise _KeyError("not found: {0} with cycle {1}\n in file: {2}".format(repr(name), cycle, self._context.sourcepath))
 
+    def close(self):
+        self._context.source.close()
+
     def __contains__(self, name):
         try:
             self.get(name)
@@ -374,7 +377,7 @@ class ROOTDirectory(object):
         return self
 
     def __exit__(self, *args, **kwds):
-        pass
+        self.close()
 
 class _KeyError(KeyError):
     def __str__(self):
@@ -459,6 +462,17 @@ def _nametitle(source, cursor):
     _endcheck(start, cursor, cnt)
     return name, title
 
+def _mapstrstr(source, cursor):
+    cursor.skip(12)
+    size = cursor.field(source, _mapstrstr._int32)
+    cursor.skip(6)
+    keys = [cursor.string(source) for i in range(size)]
+    cursor.skip(6)
+    values = [cursor.string(source) for i in range(size)]
+    return dict(zip(keys, values))
+
+_mapstrstr._int32 = struct.Struct('>I')
+
 def _readobjany(source, cursor, context, parent, asclass=None):
     # TBufferFile::ReadObjectAny()
     # https://github.com/root-project/root/blob/c4aa801d24d0b1eeb6c1623fd18160ef2397ee54/io/io/src/TBufferFile.cxx#L2684
@@ -851,6 +865,8 @@ def _defineclasses(streamerinfos, classes):
                         code.append("        cursor.skip(6)")
                         code.append("        self._{0} = cursor.array(source, cursor.field(source, self._int32), '>f8')".format(_safename(element._fName)))
                         fields.append(_safename(element._fName))
+                    elif element._fTypeName == b"map<string,string>":
+                        code.append("        self._{0} = _mapstrstr(source, cursor)".format(_safename(element._fName)))
                     else:
                         code.append("        _raise_notimplemented({0}, {1}, source, cursor)".format(repr(element.__class__.__name__), repr(repr(element.__dict__))))
                     recarray.append("raise ValueError('not a recarray')")
@@ -1602,6 +1618,28 @@ class ROOT_3a3a_TIOFeatures(ROOTStreamedObject):
 
     _format1 = struct.Struct(">B")
 
+class ROOT_3a3a_Experimental_3a3a_RNTuple(ROOTStreamedObject):
+    _classname = b"ROOT::Experimental::RNTuple"
+    classname = "ROOT::Experimental::RNTuple"
+    _fields = ["fVersion",
+               "fSize",
+               "fSeekHeader",
+               "fNBytesHeader",
+               "fLenHeader",
+               "fSeekFooter",
+               "fNBytesFooter",
+               "fLenFooter",
+               "fReserved"]
+
+    @classmethod
+    def _readinto(cls, self, source, cursor, context, parent):
+        start, cnt, self._classversion = _startcheck(source, cursor)
+        cursor.skip(4)
+        self._fVersion, self._fSize, self._fSeekHeader, self._fNBytesHeader, self._fLenHeader, self._fSeekFooter, self._fNBytesFooter, self._fLenFooter, self._fReserved = cursor.fields(source, ROOT_3a3a_Experimental_3a3a_RNTuple._format1)
+        return self
+
+    _format1 = struct.Struct(">IIQIIQIIQ")
+
 class Undefined(ROOTStreamedObject):
     _classname = None
     classname = None
@@ -1651,7 +1689,8 @@ builtin_classes = {"uproot_methods": uproot_methods,
                    "TArrayF":        TArrayF,
                    "TArrayD":        TArrayD,
                    "TRefArray":      TRefArray,
-                   "ROOT_3a3a_TIOFeatures": ROOT_3a3a_TIOFeatures}
+                   "ROOT_3a3a_TIOFeatures": ROOT_3a3a_TIOFeatures,
+                   "ROOT_3a3a_Experimental_3a3a_RNTuple": ROOT_3a3a_Experimental_3a3a_RNTuple}
 
 builtin_skip =    {"TBranch":    ["fBaskets"],
                    "TTree":      ["fUserInfo", "fBranchRef"]}
